      
"============================================================>
CLASS DEFINITION: TextPaneEnhanced
<============================================================"

Transcript cr;show: '    Filing in TextPaneEnhanced ...'.!

TextPane subclass: #TextPaneEnhanced
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''   !


!TextPaneEnhanced class methods !

supportedEvents
        "Private - Answer the Set of events that TextPanes can notify
         their owners about."
    ^super supportedEvents
        add: #cursorMoved;
        yourself! !



!TextPaneEnhanced methods !
   
characterInput: aCharacter

    super characterInput: aCharacter.
    self event: #cursorMoved. "-gmv added 11/30/92"!
   
displayChanges

    super displayChanges.
    self event: #cursorMoved. "-gmv added 11/30/92"!
   
scrollTopCornerTo: aNewCorner

    super scrollTopCornerTo: aNewCorner.
    self event: #cursorMoved. "-gmv added 11/30/92"! !


"============================================================>
CLASS DEFINITION: GrepObject
<============================================================"

Transcript cr;show: '    Filing in GrepObject ...'.!

Object subclass: #GrepObject
  instanceVariableNames:
    'directory fileName lineNumber key '
  classVariableNames: ''
  poolDictionaries: ''   !


!GrepObject class methods !

change: fromCharacter to: toCharacter in: aString

        "Utility method.
         12/5/92 -gmv.  Make a new copy
         of aString, substituting toCharacter
         for all occurrences of fromCharacter.

         NOTE: In my own system I add a method similar
         to this (minus the 'in: aString' argument)
         to the String class.   It is included here
         in modified form to minimize modifications
         made to standard Smalltalk/V classes required
         to demonstrate GrepBrowser."

    | inStream outStream nextCharacter |
    inStream := ReadStream on: aString.
    outStream := WriteStream on: ''.
    [inStream atEnd] whileFalse: [
        fromCharacter = (nextCharacter := inStream next)
        ifTrue: [outStream nextPut: toCharacter]
        ifFalse: [outStream nextPut: nextCharacter]].
    ^outStream contents.!

fromLine: line file: defaultFileName directory: defaultDirectoryName

        "Create a GrepObject from line.
         Use defaultFileName and defaultDirectoryName
         only if filename and fully qualified directory
         are not specified in line.

         Argument Types:

            line                    must be a String
            defaultFileName         must be a String, may be ''.
            defaultDirectoryName    may be a String or a Directory"


    | fileLineTextArray pathArray aGrepObject dirString fileString |

    fileLineTextArray :=  self splitLine: line fileName: defaultFileName.
    fileLineTextArray isNil ifTrue: [^nil]. "Can't parse, return error."

    aGrepObject := self new.
    defaultDirectoryName isString
    ifTrue: [
        aGrepObject directory: (self change: $/
                                     to:     $\
                                     in:     defaultDirectoryName).
    ] ifFalse: [aGrepObject directory: defaultDirectoryName].
    fileString := (self change: $/
                        to:     $\
                        in:     (fileLineTextArray at: 1)).
    "Test for drive specification in fileString"
    (((fileString at: 1) isLetter)
        & ((fileString at: 2)= $:)
        & ((fileString at: 3) ~= $\))
    ifTrue: [
        "We have a drive specifier without a fully qualified path."
        "Set drive for aGrepObject from fileString."
        (aGrepObject directory) drive: (fileString at: 1).

        "Strip drive specification from fileString."
        fileString := fileString copyFrom: 3 to: (fileString size)].

    "Use of 'File class>>splitPath:in:' could be avoided with
     a little extra code.  For now it does the job."

    pathArray := File splitPath: fileString in: (aGrepObject directory).
    dirString := ((String with: (pathArray at: 1)   "drive letter"
                          with: $:)),               "colon"
                          (pathArray at: 2).        "path name"
    ^aGrepObject
        directoryName: dirString;
        fileName:      (pathArray at: 3);
        lineNumber:    (fileLineTextArray at: 2);
        key:           line;
        yourself.!

isInteger: aString

        "Utility method.
         Test if string contains only digits"

    1 to: aString size do: [ :index |
        (aString at: index) isDigit
            ifFalse: [^false]].
    ^true.!

splitLine: line fileName: fileName

    "Scan and separate components in a grep line of the form:

        [filename:]lineNumber:text

     where [filename:] is optional.  The colons are
     literal separator characters which serve to
     break grep lines into two or three fields.

     The two output formats specified by the above
     syntax are standard for most greps when the
     -n option (include line numbers) is used.
     Generally, if more than one file is searched by
     grep, filenames appear at the beginning of the
     line.  If only a single file is searched, the
     filename is usually omitted.  Spaces are not
     allowed at the beginning of the line or before
     the text field.  This code does not test
     that a specified filename is valid.  A filename
     may include a drive designator and may include
     a partial or complete path prefix.

     Note that because the algorithm used assumes that the
     first field containing only digits is the lineNumber
     field, filenames which include only digits
     (e.g. '1' '12' '0' '12345678') will cause
     problems.  Filenames of this sort are rare,
     but even so, filenames consisting of only
     digits may be specified by appending a
     period (to indicate file extension) followed
     by an empty extension.  (e.g. '1.' '12.' '0.'
     '12345678.' name exactly the same files as above
     but are not considered line numbers because they
     contain a period).


     Algorithm:

     Split a line containing a grep specification into
     three fields.  The first two fields are terminated
     by a colon.  The first field is optional
     and represents a filename.  The second field
     is required and must be an integer representing
     a line number within the file.  If the first
     field is omitted, the argument, fileName, is
     used as the default filename.  Note that a
     filename may begin with a drive letter followed
     by a colon.  Since the scanner could legitimately
     consider this colon as a field separator, a
     modified algorithm is used.  This algorithm
     looks for the first field which contains
     an integer (only digits).  This field is considered
     to be the required lineNumber as long as it is
     within the first three fields.  Everything up to
     the colon before the lineNumber field is treated as
     the filename.  If no field precedes the lineNumber
     field, the default name is used.  Everything
     after the lineNumber field is considered the
     text (originally found by grep).  On success,
     an array of three elements is returned.
        1   filename
        2   lineNumber
        3   text
    On failure, nil is returned instead of an array."



    "==================================================>
     Start of code
    <=================================================="
    | aStream field nameField lineNumber start stop text |

    aStream := ReadStream on: line.

    "Look in first 3 fields for first field containing an integer."
    1 to: 3 do: [ :i |
        aStream skip: 1.
        start := aStream position.
        (aStream skipTo: $:) ifFalse: [^nil]. "Fail: colon not found."
        stop := (aStream position - 1).
        field := line copyFrom: start to: stop.
        (self isInteger: field) ifTrue: [
            lineNumber := field asInteger.
            nameField := line copyFrom: 1 to: start - 2.
            "If no nameField, use default filename"
            (nameField = '') ifTrue: [nameField := fileName].
            text := line copyFrom: (stop + 2) to: (line size).
            ^Array
                with: nameField
                with: lineNumber
                with: text]].
    "Failed to find lineNumber in first 3 fields. Return nil."
    ^nil! !



!GrepObject methods !

directory

    ^directory!

directory: aDirectory

    aDirectory isString
    ifTrue: [self directoryName: aDirectory]
    ifFalse: [directory := aDirectory].!

directoryName

    ^directory fullDirName.!

directoryName: aString

    directory := Directory pathName: aString.!

fileName

    ^fileName!

fileName: aString

    fileName := aString!

fullPathName

    ^self directoryName,
        self fileName.!

key

    ^key!

key: aString

    key := aString!

lineNumber

    ^lineNumber!

lineNumber: anInteger

    lineNumber := anInteger!

openFile

    | aFile aTextWindow aTextPane partial scrollLine |

    aFile := File pathName: self fullPathName.
    scrollLine := 5.

    aTextWindow := TextWindow new openOnFile: aFile.
    
    partial := aFile size > 10000. "SEE TextWindow>>openOnFile:"
    partial ifTrue: [aTextWindow readEntireFile].
    aFile close.

    "Scroll to selected line and highlight."
    aTextPane := aTextWindow pane.
    aTextPane scrollTopCornerTo:
        (1 @ ((lineNumber - scrollLine + 1) max: 1)).
    aTextPane selectBefore: 1 @ lineNumber .
    aTextPane selectTo: 0 @ (lineNumber + 1).
    aTextPane displayChanges.! !


"============================================================>
CLASS DEFINITION: GrepBrowser
<============================================================"

Transcript cr;show: '    Filing in GrepBrowser ...'.!

ViewManager subclass: #GrepBrowser
  instanceVariableNames: 
    'textPane listPane directory grepFileName textFileName 
     grepList scrollLine searchFile saveCurrentLine saveLastLine '
  classVariableNames: ''
  poolDictionaries: '' !


!GrepBrowser class methods !
 
fromCommandLine

    | commandArguments arg1
      directoryString fileString |

    commandArguments := self getCommandLineArguments.
    arg1 := self splitArgument: (commandArguments at: 1).
    directoryString := arg1 at: 1.
    fileString := arg1 at: 2.

    GrepBrowser new
        directory: directoryString;
        fileName: fileString;
        open.!
  
getCommandLineArguments

        "Split command line into arguments"

    | commandLine |
    commandLine := Smalltalk getCommandLine.
    ^commandLine asArrayOfSubstrings.!
  
splitArgument: argument "String"

        "Split command line argument into directory and
         filename components "
    | anArray directoryString fileNameString |
    anArray := File
        splitPath: argument
        in: Disk.
    directoryString := (String with: (anArray at: 1) with: $:),
        (anArray at: 2),
        ((anArray at: 2) last == $\
            ifTrue: [anArray at: 3]
            ifFalse: [(String with: $\)]).
     fileNameString := (anArray at: 3).
     ^Array with: directoryString with: fileNameString.! !
   



!GrepBrowser methods !
   
aboutGrepBrowser

    
    MessageBox message: 'GrepBrowser 2.1'!
   
accept: aPane

    self saveTextPane.!

adjustFrom: startLine
        to: stopLine
        by: adjustBy        "an Integer"
      file: fileName

        "Adjust lineNumbers in GrepObjects.  Adjustments are
         required when lines are added to or deleted from
         a file in the textPane.  There may be other GrepObjects
         in grepList which will also be affected.  The entire
         list is scanned for GrepObjects with files which
         match fileName.  Only GrepObjects with a lineNumber
         between startLine and stopLine will be modified.  The
         adjustBy argument indicates the number of lines
         added (for a positive number) or deleted (for a
         negative number)."

    | lineNumber|

    grepList do: [ :aGrepObject |
        lineNumber := aGrepObject lineNumber.

        (((fileName = aGrepObject fullPathName) and: [
          (lineNumber >= startLine) ]) and: [
          (lineNumber <= stopLine)])
        ifTrue: [
            "Adjust line number for this entry"
            aGrepObject lineNumber: lineNumber + adjustBy]].!

changeScrollLine

    "7/4/93 -gmv
     Adjust the highlight scroll line to
     be at the specified line number.

     Does NOT do bounds checking.  If you
     enter a string that can not be converted
     to a number, the asInteger message will
     convert it to 0.  You can put in totally
     ridiculous values, e.g. -1000 or 65536
     for a file that has only 100 lines.
     Such input won't break anything; it will
     simply cause the scroll line to be off
     the screen.  A scroll line larger than
     the number of lines in a file will always
     cause the first line in the file to be
     displayed when the associated list item
     is selected.  Converserly a very large
     negative number can be used if you want
     to quickly scan the last line in of each
     of the files.

     Bounds checking is easy enough to add using
     max: and min: messages.  However, I left
     out bounds checking to experiment with how
     the run-time image handles bad data on its
     own and to see if there are any interesting
     uses for a out-of-bounds scroll lines.
     See code in setTextSelection for an example
     which uses max: together with code that
     determines how many lines are in the currently
     selected file."


    | lineNumber answer |

    lineNumber := scrollLine.
    answer := Prompter prompt: 'Enter new scroll line'
        default: lineNumber printString.

    "return if dialog canceled"
    answer isNil ifTrue: [^self].

    lineNumber := answer asInteger.
    scrollLine := lineNumber.
    self setTextSelection!

checkAdjustLines

    | currentLine lastLine index newFileName
      startLine stopLine adjustBy |

        "See if line adjustment is required by
         editing changes in textPane.  Tests to
         see if a line has been added or deleted
         since the last time the cursor was moved
         by a mouse event or by a key press.


         NOTE: This method is called every time the
         cursor is moved and causes cursor movement with
         arrows to be slowed down considerably.  If you
         don't use auto-repeat keys a lot, you won't
         notice.  However, if you want to have speedy
         auto-repeat, you'll have to decide if the
         status bar and the automatic maintenance of
         lineNumber changes are worth the sacrifice.
         You can always escape into a more speedy
         standard Smalltalk file editor by double
         clicking on the desired GrepObject in the
         browser's list pane.  (Or use the List
         menu 'Open file' command.)


         BROWSING BIG FILES

         The textPane will handle files of any size.
         Files are not limited to 32K as with many
         Windows text editors.  However, larger files
         such as windows.h may take ten to fifteen
         seconds to load when first selected.  (windows.h
         is a 150K file.  30K files take 3-4 seconds
         Files 10K or less take 1-2 seconds to load.)

         Large files are not a problem if all the GrepObjects
         refer to lines in the same file.
         Files are only reloaded when you select a
         new GrepObject associated with a file that is not
         already in the textPane.  For example, once
         windows.h is loaded, you may have 1000 or
         more line number references.  As long as
         all these refer to windows.h, selecting a
         new GrepObject will not cause a reload.
         Once a file is opened, random access to any point
         in the file is instantaneous.  To see that
         this is so, change to the directory containing
         windows.h and search for 'WINVER'

            grep -n WINVER windows.h >out.grp

         Now open a GrepBrowser.

            GrepBrowser new
                directory:  'C:/borlandc/';
                fileName:   'out.grp';
                searchFile: 'windows.h';
                open.

         The first grep item you select (other than
         out.grp--the first entry in the list) will
         cause a ten second delay.  After this, you
         can scroll rapidly up and down through items
         in the list pane.  The text pane below will
         scroll and highlight lines just about as
         fast as you can move through the list.
         (Note, however, that selecting the first
         item in the list (out.grp) will
         cause windows.h to be unloaded.  It's best
         to avoid using the HOME key in the list pane
         when large files are being browsed, for this
         reason.  If you want a programming challenge,
         devise a caching mechanism that preserves
         files in memory once they have been loaded.
         HINT: use a dictionary with the full path
         name for files as a keys."

    currentLine := textPane selection origin y.     "line containg cursor"
    lastLine := textPane totalLength.               "last line in file"

    (index := listPane selection) isNil ifTrue: [^nil].
    newFileName := self getFileNameFromIndex: index.
    newFileName ~= textFileName ifTrue: [
        "Selected file has changed."
        saveCurrentLine := nil.
        saveLastLine := nil].

    "Avoid adjustment if we are here because selected file has changed."
    saveCurrentLine isNil ifTrue: [saveCurrentLine := currentLine.].
    saveLastLine    isNil ifTrue: [saveLastLine := lastLine.].

    (lastLine ~= saveLastLine) ifTrue: [
        "Line adjustment is required because there are either fewer
         or more lines in the file than the last time we checked."
        adjustBy := lastLine - saveLastLine.
        stopLine := saveLastLine.
        adjustBy > 0
            ifTrue:[startLine := saveCurrentLine + 1.]
            ifFalse: [startLine := saveCurrentLine.].

        self adjustFrom: startLine
                     to: stopLine
                     by: adjustBy        "an Integer"
                   file: textFileName].

    saveCurrentLine := currentLine.
    saveLastLine := lastLine.!
   
close: aTopPane
        "Private - Close the receiver."
    | answer |
    Smalltalk isRunTime
        ifTrue: [
            (MessageBox confirm: 'Are you sure you want to exit?')
                ifTrue: [self close. ^Smalltalk exit] ifFalse: [^self]]
        ifFalse: [^self close].!

directory

    ^directory!

directory: aDirectory

    aDirectory isString
    ifTrue: [self directoryName: aDirectory]
    ifFalse: [directory := aDirectory].!

directoryName

    ^directory fullDirName.!
   
directoryName: aString

    directory := Directory pathName:
        (GrepObject change: $/ to: $\ in: aString) asLowerCase.!

fileName

    grepFileName isNil
        ifTrue: [^'']
        ifFalse:[^grepFileName].!

fileName: aString

    grepFileName := aString asLowerCase.!
  
getFileNameFromIndex: anInteger

    | aGrepObject |

    aGrepObject := (grepList at: anInteger).
    ^aGrepObject fullPathName.!
 
getKeyStings

    grepList isNil ifTrue: [^#()].

    ^grepList collect: [ :aGrepObject |
        aGrepObject key.
    ].!

information

    | aWriteStream parentTopic
      firstVisibleLineNumber
      lastVisibleLineNumber
      selectionLineNumber
      aGrepObject |

    aWriteStream := WriteStream on: ''.


    aWriteStream nextPutAll: '
Evaluate the following code to clone
the current GrepBrowser.'; cr.
    aWriteStream nextPutAll: '
    GrepBrowser new
        directory: ''', self directoryName, ''';
        fileName: ''', self fileName, ''';
        scrollLine: ', scrollLine printString, ';
        open.',
        ''; cr.

    aWriteStream cr; nextPutAll: 'Grep List:'; cr; cr.
    2 to: grepList size do: [ :i |
        aGrepObject := grepList at: i.
        aWriteStream nextPutAll: (aGrepObject key).
    
        aWriteStream cr.
    ].

        "A loop similar to the above could be used to report
         line number changes made to original grep list."

    aWriteStream contents edit.!
   
initializeGrepList


    | aFileStream fullPathName contents |

    grepFileName isNil ifTrue: [^nil].
    fullPathName := self directoryName, grepFileName.
    (FileHandle isThere: fullPathName) ifFalse: [^nil].

    aFileStream := (File pathName: fullPathName).
    contents := fullPathName.
    contents := aFileStream contents.
    aFileStream close.
    self setGrepListFromSting: contents.!
   
initWindowSize

    ^(Display boundingBox leftTop rightAndDown: (20 @ 20))
        rightBottom: (Display width * 8 // 10 @
           (Display height * 7 // 10))!
  
inspectYourself

    "Handy for debugging."
    self inspect.!
   
listPanedoubleClickSelect: aPane

    self openSelectedFile.!
 
listPaneGetContents: aPane

    aPane contents: (self getKeyStings).!
 
listPaneGetMenu: aPane

    | aMenu |
    aPane setMenu: (aMenu := Menu new
            title: 'List';
            owner: self;
            appendItem:
        'Open file'             selector: #openSelectedFile;
            appendItem:
        'Change scroll line'    selector: #changeScrollLine;
            appendSeparator ;
            appendItem:
        'Info'                  selector: #information;
           yourself).

    Smalltalk isRunTime ifFalse: [aMenu
            appendItem:
        'Inspect'               selector: #inspectYourself].

    Smalltalk isRunTime ifTrue: [aMenu
            appendItem:
        'Reinitialize'               selector: #reinitialize].

        aMenu
            appendSeparator ;
            appendItem:
        'About'               selector: #aboutGrepBrowser.! 

   
listPaneSelect: aPane

    | index newFileName |
    index := listPane selection.
    index isNil ifTrue: [^nil].
    newFileName := self getFileNameFromIndex: index.

    "The following test acts as a cache.  IF the newly
     selected list item refers to the file which is
     already in the text pane, THEN don't generate a
     change event for the text pane and don't change
     the selected text filename"

    newFileName ~= textFileName ifTrue: [
        self changed: #textPaneGetContents:.
        textFileName := newFileName].

    self setTextSelection.
    self updateStatusPane.!
  
open

    | ratio statusPaneHeight |
    "Initialize local variables."
    ratio := 2 / 5.
    statusPaneHeight := 25.

    "Initialize GrepBrowser instance variables."
    scrollLine isNil ifTrue:[scrollLine := 5].
    searchFile isNil ifTrue:[searchFile := ''].
    textFileName := ''.             "currently selected file"
    CursorManager execute change.   "show hour glass cursor"
    self initializeGrepList.        "peform this initialization last"
    CursorManager normal change.    "show normal cursor"

    "Add subpanes, register events, map events to handlers."
    self addSubpane: (listPane := ListBox new
        owner: self;
        when: #getContents       perform: #listPaneGetContents:;
        when: #select            perform: #listPaneSelect:;
        when: #getMenu           perform: #listPaneGetMenu:;
        when: #doubleClickSelect perform: #listPanedoubleClickSelect:;
        framingBlock: ([:box|  (box leftTop) extentFromLeftTop:
            (box width @ (box height * ratio - statusPaneHeight))])).
    self addSubpane: (textPane := TextPaneEnhanced new
        owner: self;
                                    "NOTE #cursorMoved is a custom-
                                     defined event, not standard
                                     Smalltalk/V"
        when: #cursorMoved      perform: #textPaneCursorMoved:;
        when: #getContents      perform: #textPaneGetContents:;
        when: #save             perform: #accept:;
        framingBlock: ([:box| ((box leftTop) rightAndDown:
            (0 @ (box height * ratio - statusPaneHeight)))
                extentFromLeftTop:
                    (box width @ (box height * (1 - ratio))) ])).
    self addSubpane:(StatusPane new
        owner: self;
        height: statusPaneHeight;
        inset: 6;
        when: #getContents perform: #statusPaneGetContents:;
        framingBlock: [:box|
            (box leftBottom leftAndUp: (0 @ statusPaneHeight))
                rightBottom: box rightBottom]).
    "Set window title."
 
    self labelWithoutPrefix: 'Grep Browser', ': ', self directoryName, self fileName.
    self when: #close perform: #close:.
    self openWindow. "Initialization complete.  Open window."!

openSelectedFile


    | index |
    (index := listPane selection) isNil ifTrue: [^nil].
    (grepList at: index) openFile.!
   
reinitialize

    CursorManager execute change.
    textFileName := ''. "Force file to reload."
    self initializeGrepList.
    self changed:   #listPaneGetContents:
         with:      #restoreSelected.
    self changed:   #textPaneGetContents.
    CursorManager normal change.!

saveTextPane

    | index fileName aFileStream |

    textPane modified ifTrue: [
        index := listPane selection.
        index isNil ifTrue: [^nil].
        fileName := self getFileNameFromIndex: index.

        CursorManager execute change.
        aFileStream := (File pathName: fileName).
        textPane fileOutOn: aFileStream.  "See TextWindow>>saveAs"
        aFileStream close.
        textPane modified: false.
        CursorManager normal change.

        "If the grep list itself was changed,
         reinitialize the browser.  This allows
         you to add GrepObjects to the browser
         while it is running."

        index = 1 ifTrue: [self reinitialize].
    ].!
   
scrollLine

    ^scrollLine!
  
scrollLine: anInteger

    scrollLine := anInteger.!
  
searchFile

    ^searchFile.!
 
searchFile: aString

    searchFile := aString.


        "Sometimes a default filename must be
         passed as an argument to GrepObjects
         when they are first created.  The
         instance variable, searchFile,
         is used as the defaultFileName argument
         to the class method

            GrepObject class>>fromLine:file:directory:.

         If only one file was originally searched
         by grep, the name of that file will
         not normally appear in the grep's output.
         In such a case the searched file
         is referred to as the 'searchFile'
         and must be specified when the
         GrepBrowser is opened.  E.g.

            GrepBrowser new
                ...
                searchFile: 'source.cpp';
                ...
                open.

         The location of this file is relative
         to the directory which
         contains the grep data file specified
         as the grep list file.  The location of
         the target file may also be a fully
         qualified pathname attached as a
         prefix to the filename.  E.g.

            GrepBrowser new
                ...
                searchFile: 'c:/usr/src/test//app.hpp';
                ...
                open.

        If all grep output lines contain filename specifications
        the searchFile need not be specified."!
  
setGrepListFromSting: aString

        "Create collection of GrepObjects to be
         managed and displayed in listPane."

    | aReadStream line aCollection aGrepObject |
    aReadStream := ReadStream on: aString.
    aCollection := OrderedCollection new.

        "Add the grep output file itself as
         the first element in the collection
         of GrepObjects referred to by grepList."

    aCollection add: (GrepObject new
        key: grepFileName;
        fileName: grepFileName;
        directory: directory;
        lineNumber: 1).

        "For each line in aString, create a
         GrepObject from the line and add it to
         the grepList collection."

    [aReadStream atEnd] whileFalse: [
        line := aReadStream nextLine.
        line size > 0 ifTrue: [
            aCollection add: (GrepObject
                fromLine: line
                file: searchFile
                directory: directory)]].
    grepList := aCollection.!
 
setTextSelection

        "Make line visible within text pane and highlight it.
         You probably want to see lines before as well as lines
         after the highlight line.  The position of the highlight
         within the pane is controlled by the 'scrollLine' instance
         variable.  The value of this variable determines the
         the position (relative to the top of the text pane)
         to which the highlight will be scrolled.  A value of
         1 means the first line in the pane--no lines
         will be displayed before the highlight.  A value of
         5 means the hightlight will be displayed at the
         fifth line in the text pane.

         Note that the highlight may not always be positioned
         exactly at the location specified by scrollLine when
         a new GrepObject is selected.  To minimize unnecessary
         scrolling, the text pane is not scrolled if the highlight
         line is already visible within the pane when a new item
         is selected in the list pane.  For example, grep may
         find several lines in a row all matching a specified pattern.
         It is easier to see these patterns in context if the highight
         bar is moved without scrolling the pane.

         You may change the scollLine variable on the fly
         from the local menu in the list pane.  (Press the right
         mouse button in desired pane for local menus).
         The line number for the hightlight is retrieved from
         the currently seleted GrepObject in grepList."


    | index
      lineNumber
      firstVisibleLineNumber
      lastVisibleLineNumber |

    (index := listPane selection) isNil ifTrue: [^nil].
    lineNumber := (grepList at: index) lineNumber.
    lineNumber isNil ifTrue: [^nil].

    firstVisibleLineNumber := textPane topCorner y.
    lastVisibleLineNumber := firstVisibleLineNumber + (textPane charsInColumn) - 1.

    textPane selectBefore: 1 @ lineNumber .
    textPane selectTo: 0 @ (lineNumber + 1).

    "Scroll if line not visible in pane."
    (lineNumber < (firstVisibleLineNumber + 1) or:
    [lineNumber > (lastVisibleLineNumber - 1)]) ifTrue: [
        textPane scrollTopCornerTo:
            (1 @ ((lineNumber - scrollLine + 1) max: 1))].

    textPane
        displayChanges;
        showSelection.!
  
statusPaneGetContents: aStatusPane

    | statusBoxes fontWidth |
    fontWidth := aStatusPane font width.
    statusBoxes := OrderedCollection new
        add: ( StatusBox new
            space: aStatusPane font width;
            width: fontWidth * 35;
            name: #fileName);
        add: ( StatusBox new
            space: aStatusPane font width * 2;
            width: fontWidth * 5;
            name: #topLine);
        add: ( StatusBox new
            space: aStatusPane font width * 2;
            width: fontWidth * 15;
            name: #row);
        add: ( StatusBox new
            space: aStatusPane font width * 2;
            width: fontWidth * 5;
            name: #column);
        yourself.

    aStatusPane contents: statusBoxes.!
 
textPaneCursorMoved: aPane

    self updateStatusPane!

textPaneGetContents: aPane

    | anInteger aFileStream |

    "The following two variables are monitored each time
     the cursor is moved to see if lines have been added to
     or deleted from the current file.  If they are set to nil,
     the assumption is that the file has just been loaded
     and no cursor positioning has taken place.  If these
     variables are not set to nil when a new file is selected
     (by selecting a new GrepObject), items in the GrepObject
     list could have line numbers erroneously adjusted.  An
     adjustment would occur if the length of the current file
     and the length of the previous file are not the same.
     In order to avoid this problem, both saveCurrentLine and 
     saveLastLine are set to nil each time a new file is loaded
     into the text pane."

    saveCurrentLine := nil.
    saveLastLine := nil.

    anInteger := listPane selection.
    anInteger isNil ifTrue: [
        textPane contents: ''.
        ^nil].

    "The following line is redundant.  This was also done in listPaneSelect:
     which generates the event causing textPaneGetContents: to be invoked.
     textFileName is set again here for safety's sake."

    textFileName := self getFileNameFromIndex: anInteger.

    textFileName notNil ifTrue: [
        (FileHandle isThere: textFileName)  ifFalse: [
            textPane contents: 'Can''t open file: ', textFileName.
            ^nil].
        aFileStream := (File pathName: textFileName).
        textPane fileInFrom: aFileStream.

        "Close the stream to relase its file handle"
        aFileStream close].!
  
updateStatusPane

    | statusPane cursorPosition messageString
      rowString columnString topLineString
      index fileNameString |

    statusPane := self statusPane.

    cursorPosition :=
        textPane selection origin.
    rowString := cursorPosition y printString, ' / ',
        textPane totalLength printString.

    columnString := cursorPosition x printString.
    topLineString := textPane topCorner y printString.

    (index := listPane selection)
    isNil ifTrue: [
        fileNameString := ''.
    ] ifFalse: [
        fileNameString := self getFileNameFromIndex: index.
    ].


    (statusPane statusBoxAt: #fileName) contents: fileNameString.
    (statusPane statusBoxAt: #topLine) contents: topLineString.
    (statusPane statusBoxAt: #row) contents: rowString.
    (statusPane statusBoxAt: #column) contents: columnString.

    self checkAdjustLines.! !


"============================================================>
End of File
<============================================================"

