Option Explicit

Function Attempt (RetCode As Integer, ErrorMsg As String, hstmt As Long) As Integer
    If RetCode = SQL_ERROR Then
	screen.MousePointer = DEFAULT_MOUSE
	ProcessError (hstmt)
    End If
    Attempt = RetCode
End Function

Function ConvertToType (theType As Long, theStr As String) As Variant
    'Formats the data type, based on SQL Server data type, and the formatted

    Select Case theType
	Case SQL_BIT, SQL_TINYINT, SQL_SMALLINT, SQL_INTEGER
	    ConvertToType = CInt(Val(theStr))
	Case SQL_BIGINT
	    ConvertToType = CLng(Val(theStr))
	Case SQL_REAL
	    ConvertToType = CSng(Val(theStr))
	Case SQL_FLOAT
	    ConvertToType = CDbl(Val(theStr))
	Case SQL_DATE
	    ' convert date string to one VB likes
	    If Len(theStr) < 10 Then
		ConvertToType = ""
	    Else
		ConvertToType = CVDate(Mid$(theStr, 6, 2) + "/" + Mid$(theStr, 9, 2) + "/" + Mid$(theStr, 1, 4) + Mid$(theStr, 11, 9))
	    End If
	    
	Case SQL_TIME
	    If Len(theStr) < 10 Then
		ConvertToType = ""
	    Else
		ConvertToType = TimeValue(Mid$(theStr, 12, 8))
	    End If
	    
	Case SQL_CHAR, SQL_VARCHAR, SQL_LONGVARCHAR
	    ConvertToType = CStr(theStr)
	Case SQL_DECIMAL, SQL_NUMERIC
	    ConvertToType = CCur(Val(theStr))

    End Select
End Function

Sub CreateRecord (hstmt As Long, sTable As String)
    Dim i As Integer
    Dim RetCode As Integer
    Dim CNLen As Integer
    Dim NumCols As Integer
    Dim ColCode As Integer
    Dim ColScale As Integer
    Dim ColDef As Long
    Dim OutLen As Long
    Dim sBuffer As String * 256
    Dim Query As String

    Query = AddNullTerminator("select * from " & sTable & " where 1=0")

    'Query the database.
    RetCode = Attempt(SQLExecDirect(hstmt, Query, Len(Query)), "Error in executing query.", hstmt)
    If RetCode = SQL_ERROR Then Exit Sub

    'Get the number of columns.
    RetCode = Attempt(SQLNumResultCols(hstmt, NumCols), "Error in returning number of columns.", hstmt)
    If RetCode = SQL_ERROR Then Exit Sub

    'Set number of columns in record array.
    ReDim gRecord(NumCols) As DBInfo

    'Load column info into record information type.
    For i = 1 To NumCols
	RetCode = Attempt(SQLDescribeCol(hstmt, i, sBuffer, 256, CNLen, ColCode, ColDef, ColScale, 1), "Column information retrieval failed.", hstmt)
	If RetCode <> SQL_ERROR Then
	    gRecord(i).ColName = Left$(sBuffer, CNLen)
	    gRecord(i).ColType = ColCode
	Else
	    gRecord(i).ColName = ""
	    gRecord(i).ColType = 0
	End If
	If ColCode = SQL_BIT Then
	    gRecord(i).Data = 0
	Else
	    gRecord(i).Data = ""
	End If
	gRecord(i).UpdateFlag = True
    Next i

    RetCode = Attempt(SQLFreeStmt(hstmt, SQL_CLOSE), "Statement handle was not freed.", hstmt)
    If RetCode = SQL_ERROR Then Exit Sub

    gbNewRecord = True
    
End Sub

Sub GetControlInfo (hWnd As Integer)
    Dim pos As Integer
    Dim i As Integer
    Dim ReturnStr As String
    Dim sHandle As String
    Dim sPrefix As String
    
    pos = 0
    i = 0

    'Gets a HLSTR with control names and handles.
    gsWindowObjects = GetCtlHandles(hWnd)

    ReDim FieldArray(i) As FieldInfo

    'Parse the HLSTR and build the FieldArray.
    Do
	'This is the control name.
	pos = ParseFields(pos, gsWindowObjects, Chr$(9), ReturnStr)
	sPrefix = Left$(ReturnStr, 3)
	'Add to the FieldArray.
	Select Case sPrefix
	    Case "txt", "chk"
		i = i + 1
		ReDim Preserve FieldArray(i) As FieldInfo
		pos = ParseFields(pos, gsWindowObjects, Chr$(9), sHandle)
		FieldArray(i).ColName = Right$(ReturnStr, Len(ReturnStr) - 3)
		FieldArray(i).Handle = Val(sHandle)
		If sPrefix = "txt" Then
		    FieldArray(i).ControlType = "Text Box"
		ElseIf sPrefix = "chk" Then
		    FieldArray(i).ControlType = "Check Box"
		End If
	    Case Else    'Not a control that needs to be saved.
		pos = ParseFields(pos, gsWindowObjects, Chr$(9), ReturnStr)
	End Select
    Loop Until pos = 0
    
End Sub

Function GetRecValue (sColName As String) As Variant
    Dim i As Integer

    For i = 1 To UBound(gRecord)
	If gRecord(i).ColName = sColName Then
	    GetRecValue = gRecord(i).Data
	    Exit Function
	End If
    Next i
    GetRecValue = Null
End Function

Sub LoadForm (Frm As Form)
    Dim HandleWnd As Integer
    Dim i As Integer
    Dim result As Integer
    Dim sTmp As String
    Dim iTmp As Integer

    HandleWnd = Frm.hWnd
    GetControlInfo HandleWnd   'Build the FieldArray.

    For i = 1 To UBound(FieldArray)     'Put server data on the form.
	Select Case FieldArray(i).ControlType
	    Case "Check Box"
		iTmp = Val(GetRecValue(FieldArray(i).ColName))
		result = SendMessage(FieldArray(i).Handle, BM_SETCHECK, iTmp, 0&)
	    Case "Text Box"
		sTmp = GetRecValue(FieldArray(i).ColName) + Chr$(0)
		If sTmp = "#NULL#" Then
		    sTmp = "" + Chr$(0)
		End If
		result = SendMessage(FieldArray(i).Handle, WM_SETTEXT, 0, sTmp)
	End Select
    Next i
    
End Sub

Sub LoadRecord (hstmt As Long, Query As String)
    Dim i As Integer
    Dim RetCode As Integer
    Dim CNLen As Integer
    Dim NumCols As Integer
    Dim ColCode As Integer
    Dim ColScale As Integer
    Dim ColDef As Long
    Dim OutLen As Long
    Dim sBuffer As String * 256
    
    'Query the database.
    RetCode = Attempt(SQLExecDirect(hstmt, Query, Len(Query)), "Error in executing query.", hstmt)
    If RetCode = SQL_ERROR Then Exit Sub

    'Get the number of columns.
    RetCode = Attempt(SQLNumResultCols(hstmt, NumCols), "Error in returning number of columns.", hstmt)
    If RetCode = SQL_ERROR Then Exit Sub

    'Set number of records in array.
    ReDim gRecord(NumCols) As DBInfo

    'Load column info into record information type.
    For i = 1 To NumCols
	RetCode = Attempt(SQLDescribeCol(hstmt, i, sBuffer, 256, CNLen, ColCode, ColDef, ColScale, 1), "Column information retrieval failed.", hstmt)
	If RetCode <> SQL_ERROR Then
	    gRecord(i).ColName = Left$(sBuffer, CNLen)
	    gRecord(i).ColType = ColCode
	Else
	    gRecord(i).ColName = ""
	    gRecord(i).ColType = 0
	End If
    Next i

    'Load the data into record information type.
    Do While True
	RetCode = Attempt(SQLFetch(hstmt), "Row fetch failed.", hstmt) 'Get the row.
	Select Case RetCode
	    Case SQL_NO_DATA_FOUND
		Exit Do
	    Case SQL_SUCCESS
		For i = 1 To NumCols
		    RetCode = SQLGetData(hstmt, i, 1, sBuffer, 256, OutLen)
		    If RetCode = SQL_ERROR Then
			Exit Do
		    ElseIf OutLen = -1 Then
			gRecord(i).Data = ""
		    Else
			gRecord(i).Data = ConvertToType(gRecord(i).ColType, Left$(sBuffer, OutLen))
		    End If
		Next i
	    Case Else
		Exit Do
	End Select
    Loop

    RetCode = Attempt(SQLFreeStmt(hstmt, SQL_CLOSE), "Statement handle was not freed.", hstmt)
    If RetCode = SQL_ERROR Then Exit Sub

    gbNewRecord = False

End Sub

Sub LoadTable (hstmt As Long, sTable As String, Table() As Variant)
    Dim NumCols As Integer
    Dim OutLen As Long
    Dim sBuffer As String * 256
    Dim RetCode As Integer
    Dim i As Integer
    Dim j As Integer
    Dim Query As String
    Dim CNLen As Integer
    Dim ColCode As Integer
    Dim ColScale As Integer
    Dim ColDef As Long

    Query = AddNullTerminator("select * from " & sTable & " where 1=0")
    
    'Query the database.
    RetCode = Attempt(SQLExecDirect(hstmt, Query, Len(Query)), "Error in executing query.", hstmt)
    If RetCode = SQL_ERROR Then Exit Sub

    'Get the number of columns.
    RetCode = Attempt(SQLNumResultCols(hstmt, NumCols), "Error in returning number of columns.", hstmt)
    If RetCode = SQL_ERROR Then Exit Sub

    ReDim Table(NumCols, 0) As Variant
    For i = 1 To NumCols
	RetCode = Attempt(SQLDescribeCol(hstmt, i, sBuffer, 256, CNLen, ColCode, ColDef, ColScale, 1), "Column information retrieval failed.", hstmt)
	If RetCode <> SQL_ERROR Then
	    Table(i, 0) = Left$(sBuffer, CNLen)
	Else
	    Table(i, 0) = ""
	End If
    Next i
    RetCode = SQLFreeStmt(hstmt, SQL_CLOSE)
    Query = AddNullTerminator("select * from " & sTable)
    'Query the database.
    RetCode = Attempt(SQLExecDirect(hstmt, Query, Len(Query)), "Error in executing query.", hstmt)
    If RetCode = SQL_ERROR Then Exit Sub
    j = 0
    ReDim gColWidth(NumCols) As Integer
    Do While True
	RetCode = Attempt(SQLFetch(hstmt), "Query fetch failed.", hstmt)
	If RetCode = SQL_ERROR Or RetCode = SQL_SUCCESS_WITH_INFO Or RetCode = SQL_NO_DATA_FOUND Then
	    RetCode = Attempt(SQLFreeStmt(hstmt, SQL_CLOSE), "Statement handle is not freed.", hstmt)
	    Exit Do
	End If
	j = j + 1
	ReDim Preserve Table(NumCols, j) As Variant
	For i = 1 To NumCols
	    RetCode = Attempt(SQLGetData(hstmt, i, SQL_CHAR, sBuffer, 256, OutLen), "Column retrieval failed.", hstmt)
	    If OutLen = -1 Then
		Table(i, j) = ""
	    Else
		Table(i, j) = Left$(sBuffer, OutLen)
		If OutLen > gColWidth(i) Then
		    gColWidth(i) = OutLen
		End If
	    End If
	Next i
    Loop

    RetCode = SQLFreeStmt(hstmt, SQL_CLOSE)

End Sub

Sub myCloseProcess (hstmt As Long)
    Dim RetCode As Integer

    RetCode = SQLFreeStmt(hstmt, SQL_DROP)

End Sub

Function myOpenProcess () As Long
    Dim hstmt As Long

    If SQLAllocStmt(gHdbc, hstmt) <> SQL_ERROR Then
	myOpenProcess = hstmt
    Else
	myOpenProcess = SQL_ERROR
    End If

End Function

Sub ProcessError (hstmt As Long)
    Dim SQLState As String * 256
    Dim NativeError As Long
    Dim sBuffer As String * 256
    Dim OutLen As Integer
    Dim RetCode As Integer
    Dim sTmp As String
    Dim pos As Integer

    If hstmt > 0 Then
	RetCode = SQLError(gHenv, gHdbc, hstmt, SQLState, NativeError, sBuffer, 256, OutLen)
    Else
	RetCode = SQLError(gHenv, gHdbc, SQL_NULL_HSTMT, SQLState, NativeError, sBuffer, 256, OutLen)
    End If
    pos = InStr(1, SQLState, Chr$(0))
    If pos > 0 Then
	sTmp = Left$(SQLState, pos - 1)
	sTmp = Left$(sBuffer, OutLen) + Chr$(13) + "SQLState = " + sTmp + Chr$(13) + "Native Error Code = " + Str$(NativeError)
    Else
	sTmp = Left$(sBuffer, OutLen) + Chr$(13) + "SQLState = " + Chr$(13) + "Native Error Code = " + Str$(NativeError)
    End If
    MsgBox sTmp
End Sub

Sub SaveForm ()
    Dim HandleWnd As Integer
    Dim i As Integer
    Dim result As Integer
    Dim sBuffer As String * 256
    Dim sTmp As Variant
    Dim iTmp As Integer

    For i = 1 To UBound(FieldArray)     'Put server data on the form.
	Select Case FieldArray(i).ControlType
	    Case "Check Box"
		iTmp = SendMessage(FieldArray(i).Handle, BM_GETCHECK, 0, 0&)
		If iTmp <> Val(GetRecValue(FieldArray(i).ColName)) Then
		    SetRecValue FieldArray(i).ColName, iTmp
		End If
	    Case "Text Box"
		iTmp = SendMessage(FieldArray(i).Handle, WM_GETTEXT, 256, sBuffer)
		sTmp = Left(sBuffer, iTmp)
		If sTmp <> GetRecValue(FieldArray(i).ColName) Then
		    SetRecValue FieldArray(i).ColName, sTmp
		End If
	End Select
    Next i
    
End Sub

Sub SaveRecord (sTable As String, sWhere As String)
    Dim i As Integer
    Dim Query As String
    Dim Values As Variant
    Dim RetCode As Integer
    Dim hstmt As Long

    If gbNewRecord Then
	Query = "Insert " & sTable & "("
	Values = " Values("
	For i = 1 To UBound(gRecord, 1)
	    Query = Query & gRecord(i).ColName & ","
	    Select Case gRecord(i).ColType
		Case SQL_VARCHAR, SQL_CHAR, SQL_LONGVARCHAR
		    If IsNull(gRecord(i).Data) Then
			Values = Values & "null,"
		    Else
			Values = Values & "'" & gRecord(i).Data & "',"
		    End If
		Case SQL_BIT
		    If IsNull(gRecord(i).Data) Then
			Values = Values & "0,"
		    Else
			Values = Values & gRecord(i).Data & ","
		    End If
	    End Select
	Next i
	Query = Left$(Query, Len(Query) - 1) & ")"
	Values = Left$(Values, Len(Values) - 1) & ")"
	Query = Query & Values
    Else
	Values = ""
	For i = 1 To UBound(gRecord, 1)
	    If gRecord(i).UpdateFlag Then
		Values = Values & gRecord(i).ColName & "="
		Select Case gRecord(i).ColType
		    Case SQL_VARCHAR, SQL_CHAR, SQL_LONGVARCHAR
			If IsNull(gRecord(i).Data) Then
			    Values = Values & "null,"
			Else
			    Values = Values & "'" & gRecord(i).Data & "',"
			End If
		End Select
	    End If
	Next i
	If Len(Values) Then
	    Values = Left$(Values, Len(Values) - 1)
	End If
    End If

    If Not gbNewRecord Then
	Query = "Update " & sTable & " Set " & Values
    End If
    If Len(sWhere) > 0 Then
	Query = Query & " where " & sWhere
    End If
    Query = AddNullTerminator(Query)
    hstmt = myOpenProcess()
    If hstmt <> SQL_ERROR Then
	RetCode = SQLExecDirect(hstmt, Query, Len(Query))
	myCloseProcess hstmt
    Else
	MsgBox ("Unable to allocate SQL Statement handle.")
    End If

End Sub

Sub SetRecValue (sColName As String, theVariant As Variant)
    Dim i As Integer

    For i = 1 To UBound(gRecord)
	If gRecord(i).ColName = sColName Then
	    gRecord(i).Data = theVariant
	    gRecord(i).UpdateFlag = True
	    Exit Sub
	End If
    Next i
End Sub

