/*
 *      WTJDISK.C - DLL for DOS-like Disk I/O
 *      by Jim Kyle - for Windows Technical Journal Sept 1993
 *              This code may be freely used for any
 *              purpose but credit will be appreciated.
 */

#include <windows.h>
#include "wtjdisk.h"    /* declares the FSTR structure  */

/*
 *      These functions implement DOS-style interfaces for disk i/o using
 *      the standard Windows DLL functions:
 */

#pragma argsused
int FAR PASCAL LibMain( HANDLE hInstance,
                        WORD wDataSeg, WORD wHeapSize, LPSTR lpszCmdLine )
{ if( wHeapSize > 0 )
    UnlockData( 0 );
  return 1;
}

HFSTR FAR PASCAL WTJ_Open( LPSTR fnm, LPSTR mode )
{ HFILE  hfile;
  HFSTR  hmtmp = NULL;

  switch( *mode )
    {
    case 'r':
      hfile = _lopen( fnm, READ );
      if( hfile != HFILE_ERROR )
        { hmtmp = (HFSTR) GlobalAlloc( GHND, sizeof( FSTR ) );
          if( hmtmp != NULL )
            { LPFSTR lptmp = (LPFSTR) GlobalLock( hmtmp );
              lptmp->ptr = lptmp->end = 512;
              lptmp->handle = hfile;
              lptmp->mode = 'r';
              GlobalUnlock( hmtmp );
            }
          else
            _lclose( hfile );
        }
      break;
    case 'w':
      hfile = _lcreat( fnm, 0 );
      if( hfile != HFILE_ERROR )
        { hmtmp = (HFSTR) GlobalAlloc( GHND, sizeof( FSTR ) );
          if( hmtmp != NULL )
            { LPFSTR lptmp = (LPFSTR) GlobalLock( hmtmp );
              lptmp->ptr = lptmp->end = 0;
              lptmp->handle = hfile;
              lptmp->mode = 'w';
              GlobalUnlock( hmtmp );
            }
          else
            _lclose( hfile );
        }
      break;
    }
  return hmtmp;
}

DWORD FAR PASCAL WTJ_Seek( HFSTR file, DWORD loc, int x )
{ DWORD retval = 0L;
  if( file )
    { LPFSTR lpFstr = (LPFSTR) GlobalLock( (HGLOBAL) file );
      if( x == 1 )                      // adjust for data in buffer
        { if( lpFstr->mode == 'r' )     // (read mode only)
            loc -= ( lpFstr->end - lpFstr->ptr );
        }
      if( lpFstr->mode == 'w' )         // if write mode, flush buffer
        { if( lpFstr->end )
            _lwrite( lpFstr->handle, lpFstr->cBuf, lpFstr->end );
        }
      retval = _llseek( lpFstr->handle, loc, x );
      if( lpFstr->mode == 'w' )         // invalidate current buffer
        lpFstr->ptr = lpFstr->end = 0;
      else
        lpFstr->ptr = lpFstr->end = 512;
      GlobalUnlock( (HGLOBAL) file );
    }
  return retval;
}

DWORD FAR PASCAL WTJ_Tell( HFSTR file )
{ DWORD retval = 0L;
  if( file )
    retval = WTJ_Seek( file, 0L, 1 );
  return retval;
}

UINT FAR PASCAL WTJ_Read( LPSTR lpBuf, int elsz, int nbr, HFSTR file )
{ UINT retval = 0;
  nbr *= elsz;
  if( file )
    { LPFSTR lpFstr = (LPFSTR) GlobalLock( (HGLOBAL) file );
      if( lpFstr->mode == 'r' )         // must be in read mode
        while( nbr -- )
          { if( lpFstr->ptr >= lpFstr->end )
              { lpFstr->end = _lread( lpFstr->handle, lpFstr->cBuf, 512 );
                lpFstr->ptr = 0;
              }
            if( lpFstr->end )
              lpBuf[ retval++ ] = lpFstr->cBuf [ lpFstr->ptr++ ];
            else
              break;
          }
      GlobalUnlock( (HGLOBAL) file );
    }
  return retval;
}

UINT FAR PASCAL WTJ_Write( LPSTR lpBuf, int elsz, int nbr, HFSTR file )
{ UINT retval = 0;
  nbr *= elsz;
  if( file )
    { LPFSTR lpFstr = (LPFSTR) GlobalLock( (HGLOBAL) file );
      if( lpFstr->mode == 'w' )         // must be in write mode
        while( nbr -- )
          { if( lpFstr->end > 511 )
              { _lwrite( lpFstr->handle, lpFstr->cBuf, 512 );
                lpFstr->end = 0;
              }
            lpFstr->cBuf [ lpFstr->end++ ] = lpBuf[ retval++ ];
          }
      GlobalUnlock( (HGLOBAL) file );
    }
  return retval;
}

int FAR PASCAL WTJ_Getc( HFSTR file )
{ int retval = 0;
  if( file )
    { LPFSTR lpFstr = (LPFSTR) GlobalLock( (HGLOBAL) file );
      if( lpFstr->mode == 'r' )         // must be in read mode
        { if( lpFstr->ptr >= lpFstr->end )
            { lpFstr->end = _lread( lpFstr->handle, lpFstr->cBuf, 512 );
              lpFstr->ptr = 0;
            }
          if( lpFstr->end )
            retval = (int) lpFstr->cBuf [ lpFstr->ptr++ ];
        }
      GlobalUnlock( (HGLOBAL) file );
    }
  return retval;
}

int FAR PASCAL WTJ_Gets( LPSTR lpDst, int n, HFSTR file )
{ int retval = 0;
  int c;
  while( (c = 255 & WTJ_Getc( file )) != 0 )
    if( c >= ' ' || c == '\t' )
      break;
  while( c && n-- )
    { if( c < ' ' && c != '\t' )
        break;
      *lpDst++ = (char) c;
      retval++;
      c = 255 & WTJ_Getc( file );
    }
  *lpDst = '\0';
  return retval;
}

int FAR PASCAL WTJ_Putc( int c, HFSTR file )
{ int retval = 0;
  if( file )
    { LPFSTR lpFstr = (LPFSTR) GlobalLock( (HGLOBAL) file );
      if( lpFstr->mode == 'w' )         // must be in write mode
        { if( lpFstr->end > 511 )
            { _lwrite( lpFstr->handle, lpFstr->cBuf, 512 );
              lpFstr->end = 0;
            }
          lpFstr->cBuf [ lpFstr->end++ ] = c;
          retval = 1;
        }
      GlobalUnlock( (HGLOBAL) file );
    }
  return retval;
}

void FAR PASCAL WTJ_Close( HFSTR file )
{ LPFSTR lpFstr;
  if( file )
    { lpFstr = (LPFSTR) GlobalLock( (HGLOBAL) file );
      if( lpFstr->mode == 'w'           // if write mode, flush buffer
          && lpFstr->end )              // if anything is in it
            _lwrite( lpFstr->handle, lpFstr->cBuf, lpFstr->end );
      _lclose( lpFstr->handle );
      GlobalUnlock( (HGLOBAL) file );   // release file structure
      GlobalFree( (HGLOBAL) file );
    }
}

